/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
/* This example demonstrates how to transfer data to and from CASP native application running on PC.
 * It uses CASP UDIO data format through CaspUdioComm class explained in casp_udio_comm.h file
 * This example does three things
 * 1. Receives LED control from CASP native application
 * 2. Receives servo angle from CASP native application
 * 3. Measures ultra distance sensor value and sends the measured value to CASP native application with UDIO communication
 * first 4 bytes of received data: servo angle. 4 byte integer data type
 * next 4 bytes of received data: LED ON/OFF. 4 byte integer data type
 * first 4 bytes of send data: servo angle. 4 byte integer data type
 * next 4 bytes of send data: Distance sensor measured value. 4 byte integer data type
 * rest of data bytes are unused

 * This file shall be opened and compiled in Arduino IDE
*/
/* CaspUdioComm usage
 * create a global CaspUdioComm object.
 * in setup() function...
 * call Initialize() method to initialize buffers and other variables
 * call GetRecvBuffer() and GetSendBuffer() methods to get receive and send data streams respectively
 * setup three callbacks for receiving and sending serial data as shown in below code
 * in loop() function...
 * call RecvData(), IsDataRecvd(), ResetDataRecvd() and SendData() to receive and send data
 */
#include "Arduino.h"
#include "Servo.h"
#include "casp_udio_comm.h"

#define SERIAL_BAUD_RATE            115200
#define CASP_DATA_SIZE              16      //maximum is 64 bytes. limitation is due to serial interface buffer size
Servo g_servo;
CaspUdioComm g_comm;
byte *g_comm_recv_buffer, *g_comm_send_buffer;

//user variables
#define DATA_INDEX_ULTRA_DISTANCE   4
#define DATA_INDEX_LED              4
#define DATA_INDEX_SERVO            0
#define TRIGGER_PIN   3
#define ECHO_PIN      2
#define SERVO_PIN     5
#define LED_PIN       13

int comm_recvd_size_callback()
{
  return Serial.available();
}

int comm_recv_data_callback(unsigned char *rbuffer, int size_to_recv)
{
  return Serial.readBytes(rbuffer, size_to_recv);
}

int comm_send_data_callback(unsigned char *data, int data_size)
{
    Serial.write(data, data_size);
    Serial.flush();
    return data_size;
}

void setup()
{
  //parent communication setup code
  Serial.begin(SERIAL_BAUD_RATE);
  g_comm.Initialize(CASP_DATA_SIZE, CASP_DATA_SIZE, true);
  g_comm_send_buffer = g_comm.GetSendBuffer();
  g_comm_recv_buffer = g_comm.GetRecvBuffer();
  g_comm.SetDataRecvdSizeCallback(comm_recvd_size_callback);
  g_comm.SetRecvDataCallback(comm_recv_data_callback);
  g_comm.SetSendDataCallback(comm_send_data_callback);
   
  //user setup code
  g_servo.attach(SERVO_PIN);
  g_servo.write(0);
  pinMode(LED_PIN, OUTPUT);
  pinMode(TRIGGER_PIN, OUTPUT);
  pinMode(ECHO_PIN, INPUT);
}

void loop()
{
  g_comm.RecvData(millis());
  if(g_comm.IsDataRecvd())
  {
    g_comm.ResetDataRecvd();
    //set LED status
    long led_on_off = *((long*)&g_comm_recv_buffer[DATA_INDEX_LED]);
    digitalWrite(LED_PIN, led_on_off);
    
    //set servo parameters and insert back in send data stream
    long angle = *((long*)&g_comm_recv_buffer[DATA_INDEX_SERVO]);
    g_servo.write((int)angle);
    long *angle_send = (long*)&g_comm_send_buffer[DATA_INDEX_SERVO];
    *angle_send = angle;
    
    //generate pulse of around 20 micro seconds
    digitalWrite(TRIGGER_PIN, HIGH);
    delayMicroseconds(20);
    digitalWrite(TRIGGER_PIN, LOW);
    //measure echo duration and insert in the data stream
    unsigned long echo_duration = pulseIn(ECHO_PIN, HIGH, 10000);
    unsigned long *distance = (unsigned long*)&g_comm_send_buffer[DATA_INDEX_ULTRA_DISTANCE];
    *distance = echo_duration * 0.34f/2.0f;//convert to mm
    
    //send modified data
    g_comm.SendData();
  }
}
