/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/

/* This example demonstrates how to transfer data to and from CASP native application running on PC.
 * It uses CASP UDIO data format that consists of 4 byte header followed by user data. User data shall be in multiples of 4 bytes.
 * 4byte data can be 32bit integer or float. Two 4byte data can be combined to form 64bit integer or double data types.
 * Header Format: Header format shall not be modified
 * byte 0: Start byte with value 0x74
 * byte 1: user data size lower byte
 * byte 2: user data size higher byte
 * byte 3: user data checksum
 * byte 4 to end: user data

 * User data size is in bytes

 * This example does two things
 * 1. Receives servo angle from CASP native application
 * 2. Measures ultra distance sensor value and sends the measured value to CASP native application with UDIO communication
 * User Data Format (Can be modified by user):
 * first 4 bytes of received data: servo angle. 4 byte integer data type
 * first 4 bytes of send data: servo angle. 4 byte integer data type
 * second 4 bytes of send data: Distance sensor measured value. 4 byte integer data type
 * rest of data bytes are unused

 * This file shall be opened and compiled in Arduino IDE
*/

#include "Arduino.h"
#include "Servo.h"

//parent communication variables. please don't modify these values except CASP_DATA_SIZE
#define SERIAL_BAUD_RATE            115200
#define CASP_START_BYTE             0x74
#define CASP_DATA_RECV_TIMEOUT      500
#define CASP_HEADER_SIZE            4       //in bytes
#define CASP_CHECKSUM_IDX           3
#define CASP_DATA_SIZE              16      //maximum is 64 bytes
byte g_buffer[CASP_DATA_SIZE+CASP_HEADER_SIZE]; //same buffer is used for send and receive
int g_available_bytes = 0;
int g_prev_available_bytes = 0;
long g_recvd_timeout_counter = 0;
bool g_data_received = false;
Servo g_servo;

//user variables
#define DATA_INDEX_ULTRA_DISTANCE   4
#define DATA_INDEX_SERVO            0
#define TRIGGER_PIN   3
#define ECHO_PIN      2
#define SERVO_PIN     5
//
void setup()
{
  //parent communication setup code
  Serial.begin(SERIAL_BAUD_RATE);
  //user setup code
  g_servo.attach(SERVO_PIN);
  g_servo.write(0);
  pinMode(TRIGGER_PIN, OUTPUT);
  pinMode(ECHO_PIN, INPUT);
}
//
void loop()
{
  g_available_bytes = Serial.available();
  //receive data
  if(g_available_bytes == CASP_DATA_SIZE+CASP_HEADER_SIZE)
  {
    Serial.readBytes(g_buffer, g_available_bytes);
    //validate received data with checksum
    if(g_buffer[0] == CASP_START_BYTE)
    {
      byte cal_checksum = 0;
      for(byte i=0; i<CASP_DATA_SIZE; ++i)
        cal_checksum += g_buffer[CASP_HEADER_SIZE+i];
      if(cal_checksum == g_buffer[CASP_CHECKSUM_IDX])
        g_data_received = true;
    }
    g_available_bytes = 0;
  }
  else if(g_available_bytes > 0)
  {
    if(g_prev_available_bytes == g_available_bytes)
    {
      ++g_recvd_timeout_counter;
      if(g_recvd_timeout_counter > CASP_DATA_RECV_TIMEOUT)
      {
        Serial.readBytes(g_buffer, g_available_bytes);
        g_recvd_timeout_counter = 0;
        g_available_bytes = 0;
      }
    }
    else
      g_recvd_timeout_counter = 0;
  }
  g_prev_available_bytes = g_available_bytes;


  //insert user code here. current code is made synchronous with received data. it can be made asynchronous
  if(g_data_received)
  {
    //set servo parameters
    long angle = *((long*)&g_buffer[CASP_HEADER_SIZE+DATA_INDEX_SERVO]);
    g_servo.write((int)angle);
    //generate pulse of around 20 micro seconds
    digitalWrite(TRIGGER_PIN, HIGH);
    delayMicroseconds(20);
    digitalWrite(TRIGGER_PIN, LOW);
    //measure echo duration
    unsigned long *distance = (unsigned long*)&g_buffer[CASP_HEADER_SIZE+DATA_INDEX_ULTRA_DISTANCE];
    unsigned long echo_duration = pulseIn(ECHO_PIN, HIGH, 10000);
    *distance = echo_duration * 0.34f/2.0f;//convert to mm
  }


  //send modified data
  if(g_data_received)
  {
    g_buffer[0] = CASP_START_BYTE;
    g_buffer[1] = CASP_DATA_SIZE&0xff;//lsb of size
    g_buffer[2] = 0;//msb of size (always zero)
    byte cal_checksum = 0;
    for(byte i=0; i<CASP_DATA_SIZE; ++i)
      cal_checksum += g_buffer[CASP_HEADER_SIZE+i];
    g_buffer[CASP_CHECKSUM_IDX] = cal_checksum;
    Serial.write(g_buffer, CASP_HEADER_SIZE+CASP_DATA_SIZE);
    Serial.flush();
  }
  g_data_received = false;
}
